from django.core.exceptions import ValidationError
from django.db import models

DRAFT  = 'D'
ADOPTED= 'A'

class NoActiveBylaws(ValidationError): pass

class BylawsManager(models.Manager):
    def get_current_bylaws(self):
        try:
            return self.filter(status=ADOPTED)[0]
        except self.model.DoesNotExist:
            raise NoActiveBylaws('Please create an active Bylaws document')

class AdoptedManager(models.Manager):
	def get_query_set(self):
		return super(AdoptedManager, self).get_query_set().filter(status=ADOPTED)






from django.contrib import admin
from bylaws.models import *

class BylawsAdmin(admin.ModelAdmin):
    search_fields = ('content','title',)

    fieldsets = (
        (None, {'fields': ('title', 'slug', 'content', 'status', 'sites',)}),
        ('Advaneced', {
            'fields': ('markup', 'rendered_content',),
            'classes': ('collapse',)
        }),
    )

admin.site.register(Bylaws, BylawsAdmin)
admin.site.register(UserSignature)



from django.conf.urls.defaults import *
from django.views.generic.list_detail import object_detail
from bylaws.models import Bylaws, UserSignature
from bylaws.views import current_bylaws, bylaws_diff_list, bylaws_diff_detail


'''
 <root>/current/ - the latest adopted bylaws
 if admin:
     <root>/<year>-<month>/diff - list of past versions with option to compare
     <root>/<year>-<month>/diff/<version_id> - show diff of current version with history version
'''

urlpatterns = patterns('',
    url(r'^current/$', current_bylaws, name="by-bylaws-current"),
    url(r'^(?P<year_month_slug>[-\d]+)/$', bylaws_diff_list, name="by-bylaws-diff-list"),
    url(r'^(?P<year_month_slug>[-\d]+)/diff/(?P<version_id>[\d]+)/$', bylaws_diff_detail, name="by-bylaws-diff-detail"),
    url(r'^(?P<year_month_slug>[-\d]+)/diff/(?P<version_id>[\d]+)/to/(?P<second_version_id>[\d]+)/$', bylaws_diff_detail, name="by-bylaws-diff-detail"),
)




from django.shortcuts import render_to_response, get_object_or_404
from django.template.context import RequestContext

from bylaws.models import Bylaws, UserSignature

def current_bylaws(request):
    object = Bylaws.adopted_objects.latest()
    return render_to_response('bylaws/bylaws_detail.html', locals(),
                              context_instance=RequestContext(request))
    
def bylaws_list(request):
    objects = Bylaws.objects.all()
    return render_to_response('bylaws/bylaws_list.html', locals(),
                              context_instance=RequestContext(request))

def bylaws_diff_list(request, year_month_slug):
    '''
    Returns an object looked up by when it was created so that it's history can be displayed and diffed
    '''
    year, month = year_month_slug.split('-')
    object = Bylaws.objects.get(created__year=year, created__month=month)
    try:
        history= object.history.all()
    except:
        history= None 
    return render_to_response('bylaws/bylaws_diff_list.html', locals(),
                              context_instance=RequestContext(request))

def bylaws_diff_detail(request, year_month_slug, version_id, second_version_id=None):
    '''
    Returns an object looked up by when it was created so that it's history can be displayed and diffed
    '''
    year, month = year_month_slug.split('-')
    object = Bylaws.objects.get(created__year=int(year), created__month=int(month))
    first=object.history.all()[int(version_id)]
    if second_version_id:
        second=object.history.all()[int(second_version_id)]
        diff_content = first.diff(second)
    else:
        diff_content = object.diff(first)
    return render_to_response('bylaws/bylaws_diff_detail.html', locals(),
                              context_instance=RequestContext(request))




from datetime import *
from django.conf import settings
from django.db import models
from django.contrib.markup.templatetags import markup
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User

from django_extensions.db.models import TimeStampedModel
from markup_mixin.models import MarkupMixin
from simple_history.models import HistoricalRecords
from bylaws.managers import BylawsManager, AdoptedManager
from bylaws.utils.diff import textDiff

DEFAULT_DB = getattr(settings, 'DEFAULT_DB', 'default')

class Bylaws(MarkupMixin, TimeStampedModel):
    '''
    Bylaws model.
    
    Just keeps track of bylaws, plus any revisions.
    '''
    DRAFT  = 'D'
    ADOPTED= 'A'
    
    BYLAW_STATUS=(
        (DRAFT, 'Draft'),
        (ADOPTED, 'Adopted'),
    )
    title = models.CharField(max_length=30, blank=True, null=True)
    slug = models.SlugField(_('Slug'))
    status  = models.CharField(_('Status'), choices=BYLAW_STATUS, default=DRAFT, max_length=1)
    adopted_date = models.DateField(_('Adopted date'), blank=True, null=True)
    content = models.TextField(_('Content'))
    rendered_content = models.TextField(_('Rendered content'), blank=True)
    sites = models.ManyToManyField(Site)
    history = HistoricalRecords()

    objects = BylawsManager()
    adopted_objects = AdoptedManager()

    class Meta:
        verbose_name = _('Bylaws')
        verbose_name_plural = _('Bylaws')
        get_latest_by='adopted_date'
    
    class MarkupOptions:
        source_field = 'content'
        rendered_field = 'rendered_content'

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
      # TODO: Set adopted_date if status has gone from draft -> adopted
      super(Bylaws, self).save(*args, **kwargs)
    
    @models.permalink
    def get_absolute_url(self):
        return ('by-bylaws-detail', None, {'year_month_slug': self.modified.year + '-' + self.modified.month})
    
    @property
    def adopted(self): # Find the most recently adopted version of a set of bylaws
        for b in self.history.all():
            if b.status=='A':
                return b
        return None

    def diff(self, past_bylaws=None):
        if past_bylaws:
            return textDiff(self.rendered_content, past_bylaws.rendered_content)
        else:
            return self.rendered_content

class UserSignature(TimeStampedModel):
    '''User signature model
    
       Keeps track of whether a user has signed the bylaws
    '''
    bylaws=models.ForeignKey(Bylaws)
    user=models.ForeignKey(User)
    
    class Meta:
        verbose_name = _('Signature')
        verbose_name_plural = _('Signatures')
    
    def __unicode__(self):
        return u'%s has signed %s' % (self.user, self.bylaws)

    @property
    def signed_on(self):
        return self.created

    def has_user_agreed_latest_bylaws(user):
        if UserSignature.objects.filter(bylaws=Bylaws.objects.get_current_bylaws(),user=user):
            return True
        return False
